@@ -13,15 +13,17 @@
 # the License.
 
 import copy
+import hashlib
+import json
 
 from oslo_config import cfg
 from oslo_log import log
+from osprofiler import profiler
 import six
 from stevedore import driver
 
 from zaqar.common import errors
 from zaqar.common import utils
-from zaqar.i18n import _LE
 from zaqar.storage import configuration
 
 LOG = log.getLogger(__name__)
@@ -63,13 +65,11 @@ def dynamic_conf(uri, options, conf=None):
         driver_opts = utils.dict_to_conf({'message_store': storage_type})
         conf.register_opts(driver_opts, group=u'drivers')
 
-    conf.set_override('message_store', storage_type, 'drivers',
-                      enforce_type=True)
+    conf.set_override('message_store', storage_type, 'drivers')
 
     for opt in options:
         if opt in conf[storage_group]:
-            conf.set_override(opt, options[opt], group=storage_group,
-                              enforce_type=True)
+            conf.set_override(opt, options[opt], group=storage_group)
     return conf
 
 
@@ -119,7 +119,6 @@ def load_storage_driver(conf, cache, storage_type=None,
         instance to pass to the storage driver. Needed to access
         the queue controller, mainly.
     """
-
     if control_mode:
         mode = 'control'
         storage_type = storage_type or conf['drivers'].management_store
@@ -129,9 +128,9 @@ def load_storage_driver(conf, cache, storage_type=None,
 
     driver_type = 'zaqar.{0}.storage'.format(mode)
 
-    _invoke_args = [conf, cache]
+    _invoke_args = (conf, cache)
     if control_driver is not None:
-        _invoke_args.append(control_driver)
+        _invoke_args = (conf, cache, control_driver)
 
     try:
         mgr = driver.DriverManager(driver_type,
@@ -139,10 +138,17 @@ def load_storage_driver(conf, cache, storage_type=None,
                                    invoke_on_load=True,
                                    invoke_args=_invoke_args)
 
-        return mgr.driver
+        if conf.profiler.enabled:
+            if ((mode == "control" and conf.profiler.trace_management_store) or
+                    (mode == "data" and conf.profiler.trace_message_store)):
+                trace_name = '{0}_{1}_driver'.format(storage_type, mode)
+                return profiler.trace_cls(trace_name,
+                                          trace_private=True)(mgr.driver)
+        else:
+            return mgr.driver
 
     except Exception as exc:
-        LOG.error(_LE('Failed to load "{}" driver for "{}"').format(
+        LOG.error('Failed to load "{}" driver for "{}"'.format(
             driver_type, storage_type))
         LOG.exception(exc)
         raise errors.InvalidDriver(exc)
@@ -189,20 +195,47 @@ def can_connect(uri, conf=None):
     :returns: True if can connect else False
     :rtype: bool
     """
+    # NOTE(cabrera): create a mock configuration containing only
+    # the URI field. This should be sufficient to initialize a
+    # storage driver.
     conf = dynamic_conf(uri, {}, conf=conf)
     storage_type = six.moves.urllib_parse.urlparse(uri).scheme
 
     try:
-        # NOTE(cabrera): create a mock configuration containing only
-        # the URI field. This should be sufficient to initialize a
-        # storage driver.
+        ctrl = load_storage_driver(conf, None,
+                                   storage_type=conf.drivers.management_store,
+                                   control_mode=True)
         driver = load_storage_driver(conf, None,
                                      storage_type=storage_type,
-                                     control_driver=load_storage_driver
-                                     (conf, None,
-                                      storage_type=storage_type,
-                                      control_mode=True))
+                                     control_driver=ctrl)
         return driver.is_alive()
     except Exception as exc:
-        LOG.debug('Can\'t connect to: %s \n%s' % (uri, exc))
+        LOG.debug('Can\'t connect to: %s \n%s', (uri, exc))
         return False
+
+
+def get_checksum(body, algorithm='MD5'):
+    """According to the algorithm to get the message body checksum.
+
+    :param body: The message body.
+    :type body: six.text_type
+    :param algorithm: The algorithm type, default is MD5.
+    :type algorithm: six.text_type
+    :returns: The message body checksum.
+    :rtype: six.text_type
+    """
+
+    checksum = '%s:' % algorithm
+
+    if body is None:
+        return ''
+    else:
+        checksum_body = json.dumps(body).encode('utf-8')
+    # TODO(yangzhenyu): We may support other algorithms in future
+    # versions, including SHA1, SHA256, SHA512, and so on.
+    if algorithm == 'MD5':
+        md5 = hashlib.md5()
+        md5.update(checksum_body)
+        checksum += md5.hexdigest()
+
+    return checksum
